/*
 * Copyright (C) 2006 uguu@users.sourceforge.jp, All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. Neither the name of Clarkware Consulting, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without prior written permission. For written
 *       permission, please contact clarkware@clarkware.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * CLARKWARE CONSULTING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jp.sourceforge.expression_computer.command;

import jp.sourceforge.expression_computer.Command;
import jp.sourceforge.expression_computer.ComputeContext;
import jp.sourceforge.expression_computer.type.FloatingPointLiteral;

/**
 * <p>
 * スタックから2つの値をポップし、比較します。2つの値が同じである場合は1を、そうでない場合は0をスタックにプッシュします。
 * </p>
 * 
 * @author uguu@users.sourceforge.jp
 */
public final class EqualCommand implements Command {

    /**
     * {@inheritDoc}
     */
    public void execute(ComputeContext context) {
        double value2 = context.popStack().getValue(context);
        double value1 = context.popStack().getValue(context);

        double resultValue;
        if (value1 == value2) {
            resultValue = TRUE;
        } else {
            resultValue = FALSE;
        }

        context.pushStack(new FloatingPointLiteral(resultValue));
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
        return this.getClass().getName();
    }

}
