/*
 * Copyright (C) 2006 uguu@users.sourceforge.jp, All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. Neither the name of Clarkware Consulting, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without prior written permission. For written
 *       permission, please contact clarkware@clarkware.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * CLARKWARE CONSULTING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jp.sourceforge.expression_computer.node;

import jp.sourceforge.expression_computer.CompileContext;
import jp.sourceforge.expression_computer.Node;
import jp.sourceforge.expression_computer.command.PreDecrementCommand;

/**
 * <p>
 * PreDecrementExpressionを表すノードです。
 * </p>
 * 
 * @author uguu@users.sourceforge.jp
 */
public final class PreDecrementExpressionNode extends AbstractNode implements OperandNode {

    private PreDecrementExpressionNode.OperatorNode operator;

    private VariableNode                            operand;

    /**
     * <p>
     * インスタンスを初期化します。
     * </p>
     * 
     * @param operand
     *            項。<br>
     *            nullの場合、{@link NullPointerException}例外をスローします。
     */
    public PreDecrementExpressionNode(VariableNode operand) {
        if (operand == null) {
            throw new NullPointerException("operandがnullです。");
        }
        this.operator = new PreDecrementExpressionNode.OperatorNode();
        this.operand = operand;
    }

    /**
     * {@inheritDoc}
     */
    public void compile(CompileContext context) {
        this.operand.compile(context);
        this.operator.compile(context);
    }

    /**
     * {@inheritDoc}
     */
    public Node[] getChildren() {
        return new Node[] { this.operator, this.operand };
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
        return this.getClass().getName() + this.toChildrenString();
    }

    /**
     * <p>
     * PreDecrementExpressionNodeの"--"演算子を表すノードです。
     * </p>
     * 
     * @author uguu@users.sourceforge.jp
     */
    public static final class OperatorNode extends AbstractNode {

        /**
         * {@inheritDoc}
         */
        public void compile(CompileContext context) {
            context.getCommandList().add(new PreDecrementCommand());
        }

        /**
         * {@inheritDoc}
         */
        public Node[] getChildren() {
            return new Node[0];
        }

        /**
         * {@inheritDoc}
         */
        public String toString() {
            return this.getClass().getName();
        }

    }

}
